'---------------------------------------------------------------------------
'
' RC5SMX2.BAS
' RC5 Service Mode enabler eXtended version 2.01
'
' Kinzi Design, (c) 2005.
' http://www.kinzi.net/
'
'---------------------------------------------------------------------------
'
' Sending RC5 commands over IR using an AT90S2313. Starting with
' this "eXtended" version RC5-Command and RC5-Address can be set
' via PORTs.
'
' With this tool you may be able to put TV sets into service mode
' knowing the needed RC5-code without a special service-mode
' remote-control.
'
' For example, TV sets using a BEKO 14.1-chassis need RC5 0 29 to
' enable/disable service-mode.
'
'---------------------------------------------------------------------------
'
' Version 2 uses Powerdown-Mode of the C to extend battery life.
' Thus the pin assignment has changed:
'
' PORTB pins 6,5,4,2,1,0 = command (64 different commands)
' PORTD pins 5,4,3,1,0 = address (32 different adresses)
'
' PORTD pin 2 = start button (active low); makes use of the INT0-feature
'
' PORTB pin 3 = IR-LED
'
' Using Powerdown-Mode current consumption is below 0.3 A @ 3 V
' (in sleep mode) because pull-ups are switched off in sleep mode.
' Each pullup sources about 30 A when pulled down !
'
'---------------------------------------------------------------------------
'
' Using portions of code from
' RC5SEND.BAS
' (c) 2003 MCS Electronics
' code based on application note from Ger Langezaal
' +5V <---[A Led K]---[220 Ohm]---> Pb.3
' RC5SEND is using TIMER1, no interrupts are used
'
'---------------------------------------------------------------------------

$regfile = "2313def.dat"
$crystal = 3579545

Dim Togbit As Byte
Dim Command As Byte
Dim Address As Byte

Command = 0
Address = 0
Togbit = 0

Ddrd = &B00000000                                           'On Portd all pins are inputs
Portd = &B11111111                                          'Enables pull-ups on PORTD when PORTD lines are inputs

Ddrb = &B00001000                                           'Portb.3 = output, all other pins are inputs
Portb = &B11111111                                          'Enables pull-ups on PORTB when PORTB lines are inputs

Enable Interrupts
Enable Int0
On Int0 Wakeup                                              'jump to Wakeup on INT0
Config Int0 = Low Level                                     'Pulling Pind.2 low awakes CPU and starts program

Do
   Waitms 100                                               'Wait a little bit
   Portd = &B00000100                                       'Disable pull-ups, only PD.2 stays active to trigger sending
   Portb = &B00001000                                       'PB.3 is no input, has to be 1 to disable IR LEDs
   Powerdown                                                'Fall asleep
Loop

Wakeup:
   Portd = &B11111111                                       'Enables pull-ups on PORTD when PORTD lines are inputs
   Portb = &B11111111                                       'Enables pull-ups on PORTB when PORTB lines are inputs

   Toggle Togbit.5                                          'According to RC5-spec the toggle bit should be toggeled each keypress

   Command = Pinb                                           'Fetch configured command from dip-switches (pins 6,5,4,2,1,0)
   Shift Command , Right , 1                                'pins 6,5,4 -> 5,4,3
   Command.0 = Pinb.0
   Command.1 = Pinb.1
   Command.2 = Pinb.2
   Reset Command.6                                          'Ensure bits 6 and 7 are cleared because we
   Reset Command.7                                          'don't know what RC5SEND does when these bits are set

   Address = Pind                                           'Fetch configured address from dip-switches (pins 5,4,3,1,0)
   Shift Address , Right , 1                                'pins 5,4,3 -> 4,3,2
   Address.0 = Pind.0                                       'correct bits 0 and 1 due to previous shift
   Address.1 = Pind.1
   Reset Address.5                                          'Ensure bits 5 to 7 are cleared because we
   Reset Address.6                                          'don't know what RC5SEND does when these bits are set
   Reset Address.7

   Do
      Rc5send Togbit , Address , Command
      Waitms 50
   Loop Until Pind.2 = 1                                    'RC5SEND the command until start button is released

Return

End